@@ -23,11 +23,11 @@ import traceback
 import weka.core.jvm as jvm
 import weka.core.typeconv as typeconv
 import weka.core.classes as classes
-import weka.core.serialization as serialization
 from numpy import *
 from weka.core.classes import JavaObject, join_options, OptionHandler, Random, SelectedTag, Tags, Tag, JavaArray, \
     is_instance_of
 from weka.core.classes import AbstractParameter
+from weka.core.classes import serialization_write, serialization_read_all, serialization_write_all
 from weka.core.capabilities import Capabilities
 from weka.core.dataset import Instances, Instance, Attribute
 from weka.filters import Filter
@@ -59,11 +59,19 @@ class Classifier(OptionHandler):
         self.is_updateable = self.check_type(jobject, "weka.classifiers.UpdateableClassifier")
         self.is_drawable = self.check_type(jobject, "weka.core.Drawable")
         self.is_batchpredictor = self.check_type(jobject, "weka.core.BatchPredictor")
+        self._header = None
         super(Classifier, self).__init__(jobject=jobject, options=options)
-        self.__classify = javabridge.make_call(self.jobject, "classifyInstance", "(Lweka/core/Instance;)D")
-        self.__distribution = javabridge.make_call(self.jobject, "distributionForInstance", "(Lweka/core/Instance;)[D")
+
+    def _make_calls(self):
+        """
+        Method for generating instances using javabridge.make_call.
+        Members must start with "_mc_"
+        """
+        super(Classifier, self)._make_calls()
+        self._mc_classify = javabridge.make_call(self.jobject, "classifyInstance", "(Lweka/core/Instance;)D")
+        self._mc_distribution = javabridge.make_call(self.jobject, "distributionForInstance", "(Lweka/core/Instance;)[D")
         if self.is_batchpredictor:
-            self.__distributions = javabridge.make_call(
+            self._mc_distributions = javabridge.make_call(
                 self.jobject, "distributionsForInstances", "(Lweka/core/Instances;)[[D")
 
     @property
@@ -76,6 +84,16 @@ class Classifier(OptionHandler):
         """
         return Capabilities(javabridge.call(self.jobject, "getCapabilities", "()Lweka/core/Capabilities;"))
 
+    @property
+    def header(self):
+        """
+        Returns the header of the training data.
+
+        :return: the structure of the training data, None if not available
+        :rtype: Instances
+        """
+        return self._header
+
     def build_classifier(self, data):
         """
         Builds the classifier with the data.
@@ -83,6 +101,7 @@ class Classifier(OptionHandler):
         :param data: the data to train the classifier with
         :type data: Instances
         """
+        self._header = data.copy_structure()
         javabridge.call(self.jobject, "buildClassifier", "(Lweka/core/Instances;)V", data.jobject)
 
     def update_classifier(self, inst):
@@ -106,7 +125,7 @@ class Classifier(OptionHandler):
         :return: the classification (either regression value or 0-based label index)
         :rtype: float
         """
-        return self.__classify(inst.jobject)
+        return self._mc_classify(inst.jobject)
 
     def distribution_for_instance(self, inst):
         """
@@ -117,7 +136,7 @@ class Classifier(OptionHandler):
         :return: the class distribution array
         :rtype: ndarray
         """
-        pred = self.__distribution(inst.jobject)
+        pred = self._mc_distribution(inst.jobject)
         return javabridge.get_env().get_double_array_elements(pred)
 
     def distributions_for_instances(self, data):
@@ -130,7 +149,7 @@ class Classifier(OptionHandler):
         :rtype: ndarray
         """
         if self.is_batchpredictor:
-            return typeconv.jdouble_matrix_to_ndarray(self.__distributions(data.jobject))
+            return typeconv.jdouble_matrix_to_ndarray(self._mc_distributions(data.jobject))
         else:
             return None
 
@@ -235,7 +254,7 @@ class Classifier(OptionHandler):
         :rtype: tuple
         """
 
-        objs = serialization.read_all(ser_file)
+        objs = serialization_read_all(ser_file)
         if len(objs) == 1:
             return Classifier(jobject=objs[0]), None
         elif len(objs) == 2:
@@ -258,9 +277,9 @@ class Classifier(OptionHandler):
             header = Instances.template_instances(header)
 
         if header is not None:
-            serialization.write_all(ser_file, [self, header])
+            serialization_write_all(ser_file, [self, header])
         else:
-            serialization.write(ser_file, self)
+            serialization_write(ser_file, self)
 
 
 class SingleClassifierEnhancer(Classifier):
