@@ -34,6 +34,142 @@ suggestions = None
 """ dictionary for class -> package relation """
 
 
+def deepcopy(obj):
+    """
+    Creates a deep copy of the JavaObject (or derived class) or JB_Object.
+
+    :param obj: the object to create a copy of
+    :type obj: object
+    :return: the copy, None if failed to copy
+    :rtype: object
+    """
+    if isinstance(obj, JavaObject):
+        wrapped = True
+        jobject = obj.jobject
+    else:
+        wrapped = False
+        jobject = obj
+    try:
+        serialized = javabridge.make_instance("weka/core/SerializedObject", "(Ljava/lang/Object;)V", jobject)
+        jcopy = javabridge.call(serialized, "getObject", "()Ljava/lang/Object;")
+        if wrapped:
+            jcopy = obj.__class__(jobject=jcopy)
+        return jcopy
+    except JavaException as e:
+        print("Failed to create copy of " + get_classname(obj) + ": " + str(e))
+        return None
+
+
+def serialization_read(filename):
+    """
+    Reads the serialized object from disk. Caller must wrap object in appropriate Python wrapper class.
+
+    :param filename: the file with the serialized object
+    :type filename: str
+    :return: the JB_Object
+    :rtype: JB_Object
+    """
+    return javabridge.static_call(
+        "Lweka/core/SerializationHelper;", "read",
+        "(Ljava/lang/String;)Ljava/lang/Object;",
+        filename)
+
+
+def serialization_read_all(filename):
+    """
+    Reads the serialized objects from disk. Caller must wrap objects in appropriate Python wrapper classes.
+
+    :param filename: the file with the serialized objects
+    :type filename: str
+    :return: the list of JB_OBjects
+    :rtype: list
+    """
+    array = javabridge.static_call(
+        "Lweka/core/SerializationHelper;", "readAll",
+        "(Ljava/lang/String;)[Ljava/lang/Object;",
+        filename)
+    if array is None:
+        return None
+    else:
+        return javabridge.get_env().get_object_array_elements(array)
+
+
+def serialization_write(filename, jobject):
+    """
+    Serializes the object to disk. JavaObject instances get automatically unwrapped.
+
+    :param filename: the file to serialize the object to
+    :type filename: str
+    :param jobject: the object to serialize
+    :type jobject: JB_Object or JavaObject
+    """
+    if isinstance(jobject, JavaObject):
+        jobject = jobject.jobject
+    javabridge.static_call(
+        "Lweka/core/SerializationHelper;", "write",
+        "(Ljava/lang/String;Ljava/lang/Object;)V",
+        filename, jobject)
+
+
+def serialization_write_all(filename, jobjects):
+    """
+    Serializes the list of objects to disk. JavaObject instances get automatically unwrapped.
+
+    :param filename: the file to serialize the object to
+    :type filename: str
+    :param jobjects: the list of objects to serialize
+    :type jobjects: list
+    """
+    array = javabridge.get_env().make_object_array(len(jobjects), javabridge.get_env().find_class("java/lang/Object"))
+    for i in range(len(jobjects)):
+        obj = jobjects[i]
+        if isinstance(obj, JavaObject):
+            obj = obj.jobject
+        javabridge.get_env().set_object_array_element(array, i, obj)
+    javabridge.static_call(
+        "Lweka/core/SerializationHelper;", "writeAll",
+        "(Ljava/lang/String;[Ljava/lang/Object;)V",
+        filename, array)
+
+
+def to_byte_array(jobjects):
+    """
+    Serializes the list of objects into a numpy array.
+
+    :param jobjects: the list of objects to serialize
+    :type jobjects: list
+    :return: the numpy array
+    :rtype: ndarray
+    """
+    array = javabridge.get_env().make_object_array(len(jobjects), javabridge.get_env().find_class("java/lang/Object"))
+    for i in range(len(jobjects)):
+        obj = jobjects[i]
+        if isinstance(obj, JavaObject):
+            obj = obj.jobject
+        javabridge.get_env().set_object_array_element(array, i, obj)
+    return javabridge.static_call(
+        "Lweka/core/PickleHelper;", "toByteArray",
+        "([Ljava/lang/Object;)[B",
+        array)
+
+
+def from_byte_array(array):
+    """
+    Deserializes Java objects from the numpy array.
+
+    :param array: the numpy array to deserialize the Java objects from
+    :type array: ndarray
+    :return: the list of deserialized JB_Object instances
+    :rtype: list
+    """
+    byte_array = javabridge.get_env().make_byte_array(array)
+    obj_array = javabridge.static_call(
+        "Lweka/core/PickleHelper;", "fromByteArray",
+        "([B)[Ljava/lang/Object;",
+        byte_array)
+    return javabridge.get_env().get_object_array_elements(obj_array)
+
+
 def get_class(classname):
     """
     Returns the class object associated with the dot-notation classname.
@@ -525,6 +661,7 @@ class JavaObject(JSONObject):
         self.jobject = jobject
         if not has_dict_handler("JavaObject"):
             register_dict_handler("JavaObject", JavaObject.from_dict)
+        self._make_calls()
 
     def __str__(self):
         """
@@ -550,6 +687,48 @@ class JavaObject(JSONObject):
         """
         return javabridge.to_string(self.jobject)
 
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        # Remove JB_Object and replace with serialized byte array (ie numpy array)
+        del state["jobject"]
+        # Remove objects generated by javabridge.make_call
+        make_calls = []
+        for k in state.keys():
+            if k.startswith("_mc_"):
+                make_calls.append(k)
+        for k in make_calls:
+            del state[k]
+        # serialize jobject
+        state["jobject_bytes"] = None
+        if self.jobject is not None:
+            try:
+                state["jobject_bytes"] = to_byte_array([self.jobject])
+            except:
+                raise Exception("Java object is not serializable: " + get_classname(self.jobject))
+        return state
+
+    def __setstate__(self, state):
+        # Restore JB_Object instance from byte array.
+        self.__dict__.update(state)
+        self.jobject = None
+        if hasattr(self, "jobject_bytes") and (self.jobject_bytes is not None):
+            try:
+                jobjects = from_byte_array(self.jobject_bytes)
+                if len(jobjects) > 0:
+                    self.jobject = jobjects[0]
+            except:
+                pass
+        delattr(self, "jobject_bytes")
+        # restore make_calls objects
+        self._make_calls()
+
+    def _make_calls(self):
+        """
+        Method for generating instances using javabridge.make_call.
+        Members must start with "_mc_"
+        """
+        pass
+
     @property
     def classname(self):
         """
